package backend

import (
	"encoding/base64"
	"fmt"
	"image"
	"io"
	"os"

	"github.com/chai2010/webp"
	"github.com/disintegration/imaging"
	"github.com/google/uuid"
)

var ErrUnsupportedFormat error = image.ErrFormat // used internally by imaging

func SaveImage(src io.Reader, maxHeight, maxWidth int, path string) (string, error) {
	img, err := imaging.Decode(src, imaging.AutoOrientation(true))
	if err != nil {
		if err == ErrUnsupportedFormat {
			return "", ErrUnsupportedFormat
		}
		return "", fmt.Errorf("error decoding image: %v", err)
	}

	if img.Bounds().Dy() > maxHeight {
		img = imaging.Resize(img, 0, maxHeight, imaging.Lanczos)
	}
	if img.Bounds().Dx() > maxWidth {
		img = imaging.Resize(img, maxWidth, 0, imaging.Lanczos)
	}

	filename := fmt.Sprint(uuid.New(), ".webp")
	file, err := os.Create(path + filename)
	if err != nil {
		return "", fmt.Errorf("error creating new image file: %v", err)
	}
	defer file.Close()

	if err = webp.Encode(file, img, &webp.Options{Quality: 80}); err != nil {
		return "", fmt.Errorf("error encoding image as webp: %v", err)
	}

	return filename, nil
}

func ServeBase64Image(c *Config, filename string) (string, error) {
	file := c.PicsDir + "/" + filename

	img, err := os.Open(file)
	if err != nil {
		return "", fmt.Errorf("error opening file %v: %v", file, err)
	}
	defer img.Close()

	imgBytes, err := io.ReadAll(img)
	if err != nil {
		return "", fmt.Errorf("error turning %v into bytes: %v", file, err)
	}

	return base64.StdEncoding.EncodeToString(imgBytes), nil
}
